searchNodes=[{"ref":"Quetzal.html","title":"Quetzal","type":"module","doc":"Quetzal - Analytical web apps, beautiful, fast, easy and real-time using Elixir. No Javascript required. Quetzal provides easy and fast tools to make analytical web apps with real-time updates. Quetzal provides the next features: Allows create componets from Elixir code and render into views such as: graphs (plotlyjs), inputs and more. It uses a single function to allow update the components via server so instead of pulling data it is pushing data whenever you want. It tracks events from components and receives in the live view to update live view components. Using components First, define a module and use Quetzal.LiveView, you don&#39;t need mount/2 or render/1, when using the Quetzal Live View all is done: defmodule AppWeb.PieLive do use Quetzal.LiveView end With this minimal configuration Quetzal is able to render any component into the view, let&#39;s generate a pie graph to render: defmodule AppWeb.PieLive do use Quetzal.LiveView @impl Quetzal.LiveView def components(_session) do [{Quetzal.Graph, [id: &quot;mypie&quot;], [type: &quot;pie&quot;, labels: [&quot;Red&quot;, &quot;Blue&quot;], values: [10, 20]]}] end end The callback returns a new graph component and put into the view the necessary items to work with it. Live updates Now, we are going to the real-time cases, let&#39;s say we want update our pie graph when an event occurs in the server, so let&#39;s define a trigger to make it: defmodule AppWeb.PieLive do use Quetzal.LiveView @impl Quetzal.LiveView def components(_session) do [{Quetzal.Graph, [id: &quot;mypie&quot;], [type: &quot;pie&quot;, labels: [&quot;Red&quot;, &quot;Blue&quot;], values: [10, 20]]}] end def trigger_update() do :timer.sleep(5000) white = :rand.uniform(100) black = :rand.uniform(100) gray = :rand.uniform(100) components = [mypie: [labels: [&quot;Black&quot;, &quot;White&quot;, &quot;Gray&quot;], values: [black, white, gray]]] update_components(components) trigger_update() end end Let&#39;s explain the code, first to all, the trigger_update/0 can be called from iex: iex(1)&gt; AppWeb.PieLive.trigger_update Then every 5 ms a random numbers will be generated and put into values of the pie graph, and the pie graph will be updated, nice eh?. To achieve this, Quetzal uses the update_components/1 function to render the new content, also you need configure the javascript hooks, only pass the hooks into the live socket connection: ... import Quetzal from &quot;quetzal_hooks&quot; let quetzal = new Quetzal(); ... let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {hooks: quetzal.Hooks}) With this minimal configuration, we able to make a real-time app that updates the graph from the live view server. Live updates with callbacks Quetzal supports callbacks to deliver when a data has changed in a form, and then performs some update in the components, let&#39;s inspect the next code: defmodule AppWeb.LiveView do use Quetzal.LiveView, handler: __MODULE__, callbacks: [:update_output_div] @impl Quetzal.LiveView def components() do [{Quetzal.Form, [id: &quot;myform&quot;, name: &quot;myform&quot;, children: [{Quetzal.InputText, [id: &quot;mytext&quot;, value: &quot;&quot;, type: &quot;text&quot;, name: &quot;mytext&quot;]}] ]}, {Quetzal.Div, [id: &quot;mydiv&quot;, style: &quot;&quot;, children: &quot;&quot;]}] end def update_output_div(&quot;myform&quot;, &quot;mytext&quot;, [value]) do [mydiv: [children: &quot;You&#39;ve entered \#{value} value in the first input&quot;]] end end First define the handler and the callbacks, the handler is a module that will process the events, and the callbacks are a list of functions in that module, so when an events occurs then that callbacks will be called. In the components we are defining a single form with an input and a div, so when something changes in the input the live view server will send an update to the view and render the new children for the div. The callbacks receive always 3 arguments, the first is the name of the form containing the components firing the event, the second is the match against the component changed and the third is the value of all components in the form. Notes Some notes that you should be take: All setup should be similar to Phoenix Live View setup except for the first step and use Quetzal.LiveView. The hooks should be configured into your app.js file. Layouts should include plotly.js if you plan to use graphs (can be included from CDN). Ensure that quetzal hooks are included in the package.json: ... &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../deps/phoenix_live_view&quot;, &quot;quetzal_hooks&quot;: &quot;file:../../quetzal&quot; }, ... That&#39;s all, we are working to add more examples of components, inputs etc. Enjoy!."},{"ref":"Quetzal.html#child_spec/1","title":"Quetzal.child_spec/1","type":"function","doc":"Generates a child spec for Quetzal registry, it requires the name into the keywords. Example Include into your supervision tree: {Quetzal, name: Quetzal.Registry}"},{"ref":"Quetzal.Callback.html","title":"Quetzal.Callback","type":"module","doc":""},{"ref":"Quetzal.Callback.html#child_spec/1","title":"Quetzal.Callback.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Quetzal.Callback.html#start_link/1","title":"Quetzal.Callback.start_link/1","type":"function","doc":""},{"ref":"Quetzal.Component.html","title":"Quetzal.Component","type":"behaviour","doc":"The base module to make input components with custom action events You can create input components or html components from this module, using into your own definition. Use the module options to set the built-in custom tags for events or tagging html components, also you can pass keywords as options to render into the tag. Example: Create an input text to capture an string: defmodule MyComponent.Text do use Quetzal.Component, tag: &quot;input&quot; type: &quot;text&quot; end And let&#39;s use in this way: iex(10)&gt; MyComponent.Text.html_tag([id: &quot;mytext&quot;]) &quot;&lt;input id=&quot;mytext&quot; type=&quot;text&quot; phx-change=&quot;mytext&quot;&gt;&lt;/input&gt;&quot;"},{"ref":"Quetzal.Component.html#c:render/1","title":"Quetzal.Component.render/1","type":"callback","doc":""},{"ref":"Quetzal.Div.html","title":"Quetzal.Div","type":"module","doc":"Renders an html div as component Example Use from components/0 callback from your live view. {Quetzal.Div, [id: &quot;mydiv&quot;, style: &quot;&quot;, children: &quot;&quot;]}"},{"ref":"Quetzal.Div.html#html_tag/1","title":"Quetzal.Div.html_tag/1","type":"function","doc":""},{"ref":"Quetzal.Form.html","title":"Quetzal.Form","type":"module","doc":"Renders an html form as component with the change event as id. Example Use from components/0 callback from your live view. {Quetzal.Form, [id: &quot;myform&quot;, children: &quot;&quot;]}"},{"ref":"Quetzal.Form.html#html_tag/1","title":"Quetzal.Form.html_tag/1","type":"function","doc":""},{"ref":"Quetzal.Graph.html","title":"Quetzal.Graph","type":"module","doc":""},{"ref":"Quetzal.Graph.html#graph/2","title":"Quetzal.Graph.graph/2","type":"function","doc":"The base module to make graph components based on plotly.js In order to make components provide two keywords, the first one contains the div properties and the second the keywords to use as data in PlotlyJS, with this you are able to build any graph supported into PlotlyJS. Example [{Quetzal.Graph, [id: &quot;mygraph&quot;], [type: &quot;pie&quot;, values: [1,2], labels: [&quot;A&quot;, &quot;B&quot;]]}] The above code can be set as a single component in the live view to render as graph."},{"ref":"Quetzal.Graph.html#html_tag/4","title":"Quetzal.Graph.html_tag/4","type":"function","doc":""},{"ref":"Quetzal.Graph.html#js_tag/1","title":"Quetzal.Graph.js_tag/1","type":"function","doc":""},{"ref":"Quetzal.Graph.html#plotly_graph/3","title":"Quetzal.Graph.plotly_graph/3","type":"function","doc":"Returns the component that holds the graph and their data holding into a single EEx template"},{"ref":"Quetzal.InputDate.html","title":"Quetzal.InputDate","type":"module","doc":"Renders an html input date as component Example Use from components/0 callback from your live view. {Quetzal.InputDate, [id: &quot;mydate&quot;]}"},{"ref":"Quetzal.InputDate.html#html_tag/1","title":"Quetzal.InputDate.html_tag/1","type":"function","doc":""},{"ref":"Quetzal.InputText.html","title":"Quetzal.InputText","type":"module","doc":"Renders an html input text as component Example Use from components/0 callback from your live view. {Quetzal.InputText, [id: &quot;mytext&quot;]}"},{"ref":"Quetzal.InputText.html#html_tag/1","title":"Quetzal.InputText.html_tag/1","type":"function","doc":""},{"ref":"Quetzal.LiveView.html","title":"Quetzal.LiveView","type":"behaviour","doc":"Quetzal Live View provides easy interface to handle events, components and messages into Quetzal architechture in a fast and fashion way. It uses Phoenix Live View to render components and their upgrades. In order to use set the Quetzal.LiveView instead Phoenix.LiveView directly over your live module: Example: You don&#39;t need mount/2 or render/1 when using the Quetzal Live View is all done: defmodule AppWeb.PieLive do use Quetzal.LiveView end In order to render components to the live view use the components/0 callback: Example: This will render a pie graph into the view using custom components: defmodule AppWeb.PieLive do use Quetzal.LiveView @impl Quetzal.LiveView def components() do Quetzal.Graph.graph [id: &quot;mypiegraph&quot;], [type: &quot;pie&quot;, labels: [&quot;RED&quot;, &quot;BLUE&quot;], values: [20, 10]] end end You can use the graph and other components to extend your views with custom graphs, tables, etc. Also is possible upgrade the components from the server live view, use update_components/1 over your live view: Example: This example generates a random number and push to live view so Quetzal updates the component in the view: defmodule AppWeb.PieLive do use Quetzal.LiveView @impl Quetzal.LiveView def components(_session) do [{Quetzal.Graph, [id: &quot;mypie&quot;], [type: &quot;pie&quot;, labels: [&quot;Red&quot;, &quot;Blue&quot;], values: [10, 20]]}] end def trigger_update() do :timer.sleep(5000) white = :rand.uniform(100) black = :rand.uniform(100) gray = :rand.uniform(100) components = [mypie: [labels: [&quot;Black&quot;, &quot;White&quot;, &quot;Gray&quot;], values: [black, white, gray]]] update_components(components) trigger_update() end end"},{"ref":"Quetzal.LiveView.html#c:components/1","title":"Quetzal.LiveView.components/1","type":"callback","doc":""},{"ref":"Quetzal.LiveView.html#update_components/1","title":"Quetzal.LiveView.update_components/1","type":"function","doc":"Updates the components sending a message to live view, it receives components as string as in Quetzal components definition Example: components = [mypiegraph: [labels: [&quot;Black&quot;, &quot;White&quot;, &quot;Gray&quot;], values: [black, white, gray]]] update_components(components)"},{"ref":"Quetzal.Supervisor.html","title":"Quetzal.Supervisor","type":"module","doc":"A simple supervisor to start the registry and callback processes Just include into your app tree supervision: Example `{Quetzal, name: Quetzal.Registry}`"},{"ref":"Quetzal.Supervisor.html#child_spec/1","title":"Quetzal.Supervisor.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Quetzal.Supervisor.html#init/1","title":"Quetzal.Supervisor.init/1","type":"function","doc":"Callback invoked to start the supervisor and during hot code upgrades. Developers typically invoke Supervisor.init/2 at the end of their init callback to return the proper supervision flags. Callback implementation for Supervisor.init/1."},{"ref":"Quetzal.Supervisor.html#start_link/1","title":"Quetzal.Supervisor.start_link/1","type":"function","doc":""}]